use utf8;
use strict;

# サクラエディタ用キーワードファイル作成スクリプト
# 本体ver.2.26のtp_stub.hに対応。他のバージョンは未確認。

my $infile = 'tp_stub.h';
my $outfile = 'tvpcpp.kwd';
my $outfile3 = 'tvpcpp_func.khp';

my %kwd = ();
my %kwd_comp = ();

undef $/;
open (FI, $infile) or die("$infile: $!");
my $allstr = <FI>;
close (FI);
$allstr =~ s|/\*.*?\*/||gs;
my @lines = split(/\n/, $allstr);

foreach my $line (@lines) {
	$line =~ s|//.*||;
	my $k = getkwd($line);
	if ($k) {
		foreach my $x (split(/\s/, $k)) {
			$kwd{$x} = 1;
		}
	}

	my $kc = getkwdcomp($line);
	$kwd_comp{$kc} = 1;
}


my @ukwds = sort (keys %kwd);
my @ukwds_comp = sort (keys(%kwd), keys(%kwd_comp));

open (FO, ">$outfile") or die("$outfile: $!");
foreach my $k (@ukwds) {
	print FO "$k\n";
}
close (FO);


my %kwd_khp_multi = ();
foreach my $k (keys(%kwd_comp)) {
	$k =~ /(\S+)\((.*)\)/;
	my ($name, $arg) = ($1, $2);
	if (exists($kwd_khp_multi{$name})) {
		$kwd_khp_multi{$name} .= "\n$arg";
	} elsif ($name =~ /^[_a-zA-Z]/) {
		$kwd_khp_multi{$name} = $arg;
	}
}

open (FO, ">$outfile3") or die("$outfile3: $!");
my @ukwds_khp  = sort (keys(%kwd_khp_multi));
foreach my $k (@ukwds_khp) {
	my $arg = $kwd_khp_multi{$k};
	$arg = 'void' unless $arg;
	my @args = split(/\n/, $arg);
	my @lines = map { "$k\($_\)"; } (@args);
	my $help = join("\\n", @lines);

	print FO "$k /// $help\n" if ($k);
}
close (FO);


exit(0);




my $curblock = 0;

sub getkwd
{
	my ($x) = @_;
	my $rv = '';
	chomp $x;

	if ($x =~ /^inline/) {
		$x =~ /([_0-9a-zA-Z]+)\(/;
		$rv = $1;

	} elsif ($x =~ /^extern/) {
		$x =~ /([_0-9a-zA-Z]+)\(/;
		$rv = $1;

	} elsif ($x =~ /^class/) {
		$x =~ /class\s+([_a-zA-Z][_0-9a-zA-Z]+)/;
		$rv = $1;

	} elsif ($x =~ /^enum/) {
		$x =~ /^\s*enum\s+([_a-zA-Z][_0-9a-zA-Z]+)(.*)/;
		$rv = $1;
		my $next = $2;
		if ($next =~ /\{(.*)/) {
			my $next = $1;
			$rv .= getEnum($next);
		}

		$curblock = 'enum' unless ($x =~ /\};/);

	} elsif ($x =~ /^struct/) {
		$x =~ /struct\s+([_a-zA-Z][_0-9a-zA-Z]+)/;
		$rv = $1 unless ($x =~ /;$/);

	} elsif ($x =~ /^typedef/) {
		$x =~ /([_0-9a-zA-Z]+)\s*;/;
		$rv = $1;

	} elsif ($x =~ /^\s*#define/) {
		$x =~ /#define\s+([_a-zA-Z][_0-9a-zA-Z]+)/;
		$rv = $1;

	} elsif ($curblock eq 'enum') {
		$rv .= getEnum($x);
	}

	if ($x =~ /^\};/) {
		$curblock = 0;
	}


if ($rv && ($rv =~ /enum/)) {
	print "$rv\n";
}

	return $rv;
}

# 補完用
sub getkwdcomp
{
	my ($x) = @_;
	my $rv = '';
	chomp $x;

	if ($x =~ s/^inline\s+(.*)//) {
		$rv = $1;
		$rv = '' unless ($rv =~ /\(/);

	} elsif ($x =~ s/^extern\s+(.*)//) {
		$rv = $1;
		$rv = '' unless ($rv =~ /\(/);

	}

	$rv =~ s/\s,/,/g;

	return $rv;
}


sub getEnum
{
	my ($x) = @_;
	my $rv = '';
	$x =~ s/\s*$//;
	$x =~ s/\{//;
	$x =~ s/\};//;
	my @xs = split(/,/, $x);

	foreach my $y (@xs) {
		my ($lhs, $rhs) = split(/=/, $y);
		$rv .= ' ' . $lhs if ($lhs);
	}

	return $rv;
}


